# COPYRIGHT (C) 2020-2022 Nicotine+ Contributors
# COPYRIGHT (C) 2008-2010 quinox <quinox@users.sf.net>
#
# GNU GENERAL PUBLIC LICENSE
#    Version 3, 29 June 2007
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from random import choice
from pynicotine.pluginsystem import BasePlugin, ResponseThrottle


class Plugin(BasePlugin):

    def __init__(self, *args, **kwargs):

        super().__init__(*args, **kwargs)

        self.settings = {
            'replies': ['Test failed.']
        }
        self.metasettings = {
            'replies': {
                'description': 'Replies:',
                'type': 'list string'
            }
        }

        self.throttle = ResponseThrottle(self.core, self.human_name)

    def incoming_public_chat_event(self, room, user, line):

        if line.lower() != 'test':
            return

        if self.throttle.ok_to_respond(room, user, line):
            self.throttle.responded()
            self.send_public(room, choice(self.settings['replies']).lstrip("!"))



# COPYRIGHT (C) 2021-2022 Nicotine+ Contributors
#
# GNU GENERAL PUBLIC LICENSE
#    Version 3, 29 June 2007
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import unittest

from collections import deque

from pynicotine.config import config
from pynicotine.search import Search
from pynicotine.slskmessages import increment_token

SEARCH_TEXT = '70 gwen "test" -mp3 -nothanks a:b;c+d +++---}[ *ello [[ @@ auto -no yes'
SEARCH_MODE = 'global'


class SearchTest(unittest.TestCase):

    def setUp(self):

        config.data_dir = os.path.dirname(os.path.realpath(__file__))
        config.filename = os.path.join(config.data_dir, "temp_config")

        config.load_config()

        self.search = Search(None, config, deque(), None, None)

    def test_do_search(self):
        """ Test the do_search function, including the outgoing search term and search history """

        old_token = self.search.token

        # Try a search with special characters removed

        config.sections["searches"]["remove_special_chars"] = True
        search_term, search_term_without_special, *_unused = self.search.process_search_term(SEARCH_TEXT, SEARCH_MODE)
        self.search.do_search(SEARCH_TEXT, SEARCH_MODE)

        self.assertEqual(self.search.token, old_token + 1)
        self.assertEqual(search_term, "70 gwen test a b c d auto yes -mp3 -nothanks *ello -no")
        self.assertEqual(search_term_without_special, "70 gwen test a b c d auto yes")
        self.assertEqual(config.sections["searches"]["history"][0], search_term)

        # Try a search without special characters removed

        config.sections["searches"]["remove_special_chars"] = False
        search_term, search_term_without_special, *_unused = self.search.process_search_term(SEARCH_TEXT, SEARCH_MODE)
        self.search.do_search(SEARCH_TEXT, SEARCH_MODE)

        self.assertEqual(search_term, '70 gwen "test" a:b;c+d +++---}[ [[ @@ auto yes -mp3 -nothanks *ello -no')
        self.assertEqual(search_term_without_special, '70 gwen "test" a:b;c+d +++---}[ [[ @@ auto yes')
        self.assertEqual(config.sections["searches"]["history"][0], search_term)
        self.assertEqual(config.sections["searches"]["history"][1],
                         "70 gwen test a b c d auto yes -mp3 -nothanks *ello -no")

    def test_search_token_increment(self):
        """ Test that search token increments work properly """

        old_token = self.search.token

        self.search.token = increment_token(self.search.token)
        self.assertEqual(old_token, self.search.token - 1)

    def test_wishlist_add(self):
        """ Test that items are added to the wishlist properly """

        old_token = self.search.token

        # First item

        self.search.add_wish(SEARCH_TEXT)
        self.assertEqual(config.sections["server"]["autosearch"][0], SEARCH_TEXT)
        self.assertEqual(self.search.token, old_token + 1)
        self.assertEqual(self.search.token, self.search.token)

        # Second item

        new_item = SEARCH_TEXT + "1"
        self.search.add_wish(new_item)
        self.assertEqual(config.sections["server"]["autosearch"][0], SEARCH_TEXT)
        self.assertEqual(config.sections["server"]["autosearch"][1], new_item)

    def test_create_search_result_list(self):
        """ Test creating search result lists from the word index """

        word_index = {
            "iso": [34, 35, 36, 37, 38],
            "lts": [63, 68, 73],
            "system": [37, 38],
            "linux": [35, 36]
        }

        search_term = "linux game iso stem"
        excluded_words = ["linux", "game"]
        partial_words = ["stem"]

        results = self.search.create_search_result_list(search_term, word_index, excluded_words, partial_words)
        self.assertEqual(results, {37, 38})

        search_term = "linux game lts music iso cd"
        excluded_words = ["linux", "game", "music", "cd"]
        partial_words = []

        results = self.search.create_search_result_list(search_term, word_index, excluded_words, partial_words)
        self.assertEqual(results, set())



# COPYRIGHT (C) 2021-2022 Nicotine+ Contributors
#
# GNU GENERAL PUBLIC LICENSE
#    Version 3, 29 June 2007
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import unittest

from collections import deque
from collections import OrderedDict
from unittest.mock import MagicMock
from unittest.mock import Mock

from pynicotine.config import config
from pynicotine.userbrowse import UserBrowse
from pynicotine.transfers import Transfers


class TransfersTest(unittest.TestCase):

    def setUp(self):

        config.data_dir = os.path.dirname(os.path.realpath(__file__))
        config.filename = os.path.join(config.data_dir, "temp_config")

        config.load_config()
        config.sections["transfers"]["downloaddir"] = config.data_dir

        self.transfers = Transfers(MagicMock(), config, deque(), Mock())
        self.transfers.init_transfers()
        self.transfers.server_login()
        self.transfers.allow_saving_transfers = False

        self.userbrowse = UserBrowse(MagicMock(), config, Mock())
        self.userbrowse.core.transfers = self.transfers

    def test_load_downloads(self):
        """ Test loading a downloads.json file """

        self.assertEqual(len(self.transfers.queue), 2)
        self.assertEqual(len(self.transfers.downloads), 13)

        transfer = self.transfers.downloads[0]

        self.assertEqual(transfer.user, "user13")
        self.assertEqual(transfer.filename, "Downloaded\\Song13.mp3")
        self.assertEqual(transfer.status, "User logged off")
        self.assertEqual(transfer.size, 0)
        self.assertIsNone(transfer.current_byte_offset)
        self.assertIsNone(transfer.bitrate)
        self.assertIsNone(transfer.length)

        transfer = self.transfers.downloads[12]

        self.assertEqual(transfer.user, "user1")
        self.assertEqual(transfer.filename, "Downloaded\\Song1.mp3")
        self.assertEqual(transfer.status, "Paused")
        self.assertEqual(transfer.size, 10093741)
        self.assertEqual(transfer.current_byte_offset, 5000)
        self.assertEqual(transfer.bitrate, "320")
        self.assertEqual(transfer.length, "4:12")

    def test_save_downloads(self):
        """ Verify that the order of the download list at the end of the session
        is identical to the one we loaded. Ignore transfer 13, since its missing
        properties will be added at the end of the session. """

        self.transfers.abort_transfers()

        old_transfers = self.transfers.load_transfers_file(self.transfers.downloads_file_name)[:12]

        saved_transfers = self.transfers.get_downloads()[:12]
        self.assertEqual(old_transfers, saved_transfers)

    def test_load_uploads(self):
        """ Test loading a uploads.json file """

        # Only finished uploads are loaded, other types should never be stored
        self.assertEqual(len(self.transfers.uploads), 3)
        self.assertEqual(len(self.transfers.queue), 2)

        transfer = self.transfers.uploads[0]

        self.assertEqual(transfer.user, "user5")
        self.assertEqual(transfer.filename, "Junk\\Song5.mp3")
        self.assertEqual(transfer.status, "Finished")
        self.assertEqual(transfer.size, 11733776)
        self.assertEqual(transfer.current_byte_offset, 11733776)
        self.assertIsNone(transfer.bitrate)
        self.assertIsNone(transfer.length)

        transfer = self.transfers.uploads[2]

        self.assertEqual(transfer.user, "user3")
        self.assertEqual(transfer.filename, "Junk\\Song3.flac")
        self.assertEqual(transfer.status, "Finished")
        self.assertEqual(transfer.size, 27231044)
        self.assertEqual(transfer.current_byte_offset, 27231044)
        self.assertEqual(transfer.bitrate, "792")
        self.assertEqual(transfer.length, "4:28")

    def test_queue_download(self):
        """ Verify that new downloads are prepended to the list """

        self.transfers.get_file("newuser", "Hello\\Path\\File.mp3", "")
        transfer = self.transfers.downloads[0]

        self.assertEqual(transfer.user, "newuser")
        self.assertEqual(transfer.filename, "Hello\\Path\\File.mp3")
        self.assertEqual(transfer.path, "")

    def test_push_upload(self):
        """ Verify that new uploads are prepended to the list """

        self.transfers.push_file("newuser2", "Hello\\Upload\\File.mp3", 2000, "/home/test")
        self.transfers.push_file("newuser99", "Home\\None.mp3", 100, "/home/more")
        transfer = self.transfers.uploads[1]

        self.assertEqual(transfer.user, "newuser2")
        self.assertEqual(transfer.filename, "Hello\\Upload\\File.mp3")
        self.assertEqual(transfer.path, "/home/test")

    def test_incomplete_download_path(self):
        """ Verify that the basename in incomplete download paths doesn't exceed 255 bytes.
        The basename can be shorter than 255 bytes when a truncated multi-byte character is discarded. """

        user = "abc"
        incomplete_folder = "incomplete_downloads"

        # Short file extension
        virtual_path = ("Music\\Test\\片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片"
                        + "片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片.mp3")
        incomplete_path = self.transfers.get_incomplete_file_path(incomplete_folder, user, virtual_path)

        self.assertEqual(len(os.path.basename(incomplete_path).encode('utf-8')), 253)
        self.assertEqual(
            incomplete_path,
            os.path.join(
                incomplete_folder,
                ("INCOMPLETEeded5d7eb6768cac99e7575549a45126片片片片片片片片片片片片片片片片片片片片片片片片"
                 "片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片.mp3")
            )
        )

        # Long file extension
        virtual_path = ("Music\\Test\\abc123456.片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片"
                        + "片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片")
        incomplete_path = self.transfers.get_incomplete_file_path(incomplete_folder, user, virtual_path)

        self.assertEqual(len(os.path.basename(incomplete_path).encode('utf-8')), 253)
        self.assertEqual(
            incomplete_path,
            os.path.join(
                incomplete_folder,
                ("INCOMPLETEcc5054eeb2a488b3a0287fda4c938ef2.片片片片片片片片片片片片片片片片片片片片片片片"
                 "片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片片")
            )
        )

    def test_download_folder_destination(self):
        """ Verify that the correct download destination is used """

        user = "newuser"
        folder = "Hello\\Path"
        destination_default = self.transfers.get_folder_destination(user, folder)

        self.transfers.requested_folders[user][folder] = "test"
        destination_custom = self.transfers.get_folder_destination(user, folder)

        config.sections["transfers"]["usernamesubfolders"] = True
        destination_user = self.transfers.get_folder_destination(user, folder)

        folder = "Hello"
        destination_root = self.transfers.get_folder_destination(user, folder)

        folder = "Hello\\Path\\Depth\\Test"
        destination_depth = self.transfers.get_folder_destination(user, folder)

        self.assertEqual(destination_default, os.path.join(config.data_dir, "Path"))
        self.assertEqual(destination_custom, os.path.join("test", "Path"))
        self.assertEqual(destination_user, os.path.join(config.data_dir, "newuser", "Path"))
        self.assertEqual(destination_root, os.path.join(config.data_dir, "newuser", "Hello"))
        self.assertEqual(destination_depth, os.path.join(config.data_dir, "newuser", "Test"))

    def test_download_subfolders(self):
        """ Verify that subfolders are downloaded to the correct location """

        user = "random"
        target_folder = "share\\Soulseek"

        shares_list = OrderedDict([
            ('share\\Music', [
                (1, 'music1.mp3', 1000000, '', {}),
                (1, 'music2.mp3', 2000000, '', {})
            ]),
            ('share\\Soulseek', [
                (1, 'file1.mp3', 3000000, '', {}),
                (1, 'file2.mp3', 4000000, '', {})
            ]),
            ('share\\Soulseek\\folder1', [
                (1, 'file3.mp3', 5000000, '', {})
            ]),
            ('share\\Soulseek\\folder1\\folder', [
                (1, 'file4.mp3', 6000000, '', {})
            ]),
            ('share\\Soulseek\\folder2', [
                (1, 'file5.mp3', 7000000, '', {})
            ]),
            ('share\\Soulseek\\folder2\\folder3', [
                (1, 'file6.mp3', 8000000, '', {})
            ])
        ])

        self.transfers.downloads.clear()
        self.userbrowse.download_folder(user, target_folder, shares_list, prefix="test", recurse=True)

        self.assertEqual(len(self.transfers.downloads), 6)

        self.assertEqual(self.transfers.downloads[0].path, os.path.join("test", "Soulseek", "folder2", "folder3"))
        self.assertEqual(self.transfers.downloads[1].path, os.path.join("test", "Soulseek", "folder2"))
        self.assertEqual(self.transfers.downloads[2].path, os.path.join("test", "Soulseek", "folder1", "folder"))
        self.assertEqual(self.transfers.downloads[3].path, os.path.join("test", "Soulseek", "folder1"))
        self.assertEqual(self.transfers.downloads[4].path, os.path.join("test", "Soulseek"))
        self.assertEqual(self.transfers.downloads[5].path, os.path.join("test", "Soulseek"))



#!/usr/bin/env python3
# COPYRIGHT (C) 2020-2022 Nicotine+ Contributors
# COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
# COPYRIGHT (C) 2009-2010 quinox <quinox@users.sf.net>
# COPYRIGHT (C) 2009 hedonist <ak@sensi.org>
# COPYRIGHT (C) 2006-2009 daelstorm <daelstorm@gmail.com>
# COPYRIGHT (C) 2008-2009 eLvErDe <gandalf@le-vert.net>
#
# GNU GENERAL PUBLIC LICENSE
#    Version 3, 29 June 2007
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
    To install Nicotine+ on a GNU/Linux distribution, run:
    pip3 install .
"""

import glob

from setuptools import find_packages
from setuptools import setup
from setuptools.command.build_py import build_py

from pynicotine.config import config
from pynicotine.i18n import build_translations
from pynicotine.i18n import get_translation_paths


class BuildPyCommand(build_py):

    def run(self):
        build_translations()
        build_py.run(self)


if __name__ == '__main__':

    setup(
        name="nicotine-plus",
        version=config.version,
        license="GPLv3+",
        description="Graphical client for the Soulseek peer-to-peer network",
        long_description="""Nicotine+ is a graphical client for the Soulseek peer-to-peer
network.

Nicotine+ aims to be a pleasant, free and open source (FOSS)
alternative to the official Soulseek client, providing additional
functionality while keeping current with the Soulseek protocol.""",
        author=config.author,
        author_email="nicotine-team@lists.launchpad.net",
        url=config.website_url,
        platforms="any",
        classifiers=[
            "Development Status :: 5 - Production/Stable",
            "Environment :: X11 Applications :: GTK",
            "Intended Audience :: End Users/Desktop",
            "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
            "Operating System :: OS Independent",
            "Programming Language :: Python",
            "Topic :: Communications :: Chat",
            "Topic :: Communications :: File Sharing",
            "Topic :: Internet",
            "Topic :: System :: Networking"
        ],
        packages=find_packages(include=["pynicotine", "pynicotine.*"]),
        package_data={"": ["*.bin", "*.ui", "PLUGININFO"]},
        scripts=["nicotine"],
        data_files=[
            ("share/applications", ["data/%s.desktop" % config.application_id]),
            ("share/metainfo", ["data/%s.appdata.xml" % config.application_id]),
            ("share/man/man1", ["data/nicotine.1"]),
            ("share/icons/hicolor/16x16/apps",
                ["pynicotine/gtkgui/icons/hicolor/16x16/apps/%s.png" % config.application_id]),
            ("share/icons/hicolor/24x24/apps",
                ["pynicotine/gtkgui/icons/hicolor/24x24/apps/%s.png" % config.application_id]),
            ("share/icons/hicolor/32x32/apps",
                ["pynicotine/gtkgui/icons/hicolor/32x32/apps/%s.png" % config.application_id]),
            ("share/icons/hicolor/48x48/apps",
                ["pynicotine/gtkgui/icons/hicolor/48x48/apps/%s.png" % config.application_id]),
            ("share/icons/hicolor/64x64/apps",
                ["pynicotine/gtkgui/icons/hicolor/64x64/apps/%s.png" % config.application_id]),
            ("share/icons/hicolor/128x128/apps",
                ["pynicotine/gtkgui/icons/hicolor/128x128/apps/%s.png" % config.application_id]),
            ("share/icons/hicolor/256x256/apps",
                ["pynicotine/gtkgui/icons/hicolor/256x256/apps/%s.png" % config.application_id]),
            ("share/icons/hicolor/scalable/apps",
                ["pynicotine/gtkgui/icons/hicolor/scalable/apps/%s.svg" % config.application_id]),
            ("share/icons/hicolor/symbolic/apps",
                ["pynicotine/gtkgui/icons/hicolor/symbolic/apps/%s-symbolic.svg" % config.application_id]),
            ("share/icons/hicolor/scalable/intl", glob.glob("pynicotine/gtkgui/icons/hicolor/scalable/intl/*.svg")),
            ("share/icons/hicolor/scalable/status", glob.glob("pynicotine/gtkgui/icons/hicolor/scalable/status/*.svg"))
        ] + get_translation_paths(),
        python_requires=">=3.5",
        install_requires=["PyGObject>=3.18"],
        cmdclass={"build_py": BuildPyCommand}
    )
